// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef AbstractGraph_H
#define AbstractGraph_H

#include <malloc.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <vector>
#include <set>

#include "DataLayer/FactorNode.h"

class AbstractGraph{
	protected:
		unsigned int edgeCount;
		unsigned int nodeCount;
		unsigned int Fcount;
		unsigned int delta_max;
		unsigned int nodesInFactors;
		unsigned int allocated;
		unsigned int maxNeighbours;

		unsigned int maxNodes;
		unsigned int maxEdges;
		unsigned int Fmax;

		std::vector<std::set<unsigned int>* > adjacency;

		unsigned int* edges;
		unsigned int* edgeMap;
		unsigned long* nodeFactors;
		unsigned long* nodeFactorsIdx;
		unsigned int* nodeFactorsMap;
		FactorNode* factors;

		bool updateEdges;
		bool updateEdgeMap;

	public:

		static const unsigned int pbase = 2063; // nextPrime(n);
		static const unsigned int ybase = 67; // nextPrime(|Y|);

		AbstractGraph(){
			edgeCount		= 0;
			nodeCount		= 0;
			Fcount		= 0;
			nodesInFactors	= 0;
			delta_max		= 0;
			allocated		= 0;
			maxNodes		= 0;
			maxEdges		= 0;
			Fmax		= 0;
			maxNeighbours	= 0;
			updateEdges		= false;
			updateEdgeMap	= false;
		};

		virtual bool isDynamic() = 0;
		virtual void setNodeCount(unsigned int) = 0;
		virtual void addNode() = 0;
		virtual void rebuildFactors() = 0;

		bool isNode(unsigned int v){
			return (v < getNodeCount());
		}

		unsigned int getNodeCount(){
			return nodeCount;
		}

		unsigned int getFactorCount(){
			return Fcount;
		}

		unsigned int getmaxNeighbours(){
			return maxNeighbours;
		}

		unsigned int getDeltaMax(){
			return delta_max;
		}

		unsigned int getNodesInFactors(){
			return nodesInFactors;
		}

		virtual unsigned int getTemplateCount() = 0;

		virtual void addEdge(unsigned int u, unsigned int v, bool undirected = true) = 0;
		virtual bool isEdge(unsigned int u, unsigned int v) = 0;

		static unsigned long baseEncode(unsigned int *a, unsigned int n, unsigned int b){
			unsigned long result = 0;
			unsigned int p = 1;

			for(unsigned int i=0; i<n; i++){
				result += a[i]*p;
				p *= b;
			}

			return result;
		}

		static unsigned long baseEncode(unsigned int a0, unsigned int a1, unsigned int b){
			unsigned long result = 0;
			unsigned int p = 1;

			//for(unsigned int i=0; i<n; i++){
				result += a0*p;
				p *= b;
				result += a1*p;
			//	p *= b;
			//}

			return result;
		}

		static unsigned long baseEncode(unsigned int a0, unsigned int a1, unsigned int a2, unsigned int b){
			unsigned long result = 0;
			unsigned int p = 1;

			//for(unsigned int i=0; i<n; i++){
				result += a0*p;
				p *= b;
				result += a1*p;
				p *= b;
				result += a2*p;
			//	p *= b;
			//}

			return result;
		}

		static void baseDecode(unsigned int *a, unsigned int n, unsigned int b, unsigned long m){
			unsigned long c = m;

			for(unsigned int i=0; i<n; i++){
				a[i] = c%b;
				c    = c/b;
			}
		}

		static void baseDecode(unsigned int& a0, unsigned int& a1, unsigned int b, unsigned long m){
			unsigned long c = m;

			a0 = c%b;
			c  = c/b;
			a1 = c%b;
		}

		unsigned int getFactor(unsigned int* A, unsigned long m){

			unsigned int i = 0;
			AbstractGraph::baseDecode(A, delta_max, AbstractGraph::pbase, m);
			for(; A[i] && i<delta_max; i++) A[i]--;
			return i+1;
		}

		unsigned int getAllocated(){
			return allocated;
		}

		unsigned int getMaxNodes(){
			return maxNodes;
		}

		unsigned int getMaxEdges(){
			return maxEdges;
		}

		unsigned int getFmax(){
			return Fmax;
		}

		unsigned int* getEdges(){
			return edges;
		}

		unsigned int* getEdgeMap(){
			return edgeMap;
		}

		FactorNode* getFactors(){
			return factors;
		}

		unsigned long* getNodeFactors(){
			return nodeFactors;
		}

		unsigned int* getNodeFactorsMap(){
			return nodeFactorsMap;
		}

		unsigned long* getNodeFactorsIdx(){
			return nodeFactorsIdx;
		}
};
#endif
