// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef Model_H
#define Model_H

#include <malloc.h>

#include <string>

#include "AbstractGraph.h"
#include "AttributeConsumer.h"
#include "DataLayer/ObservationSet.h"

template <class T>
class Model : public AttributeConsumer<T>{
	protected:
		AbstractGraph* graph;
		T* parameters;
		unsigned int length;
		float init_stepsize;
		float init_weight;

	public:
		Model() : AttributeConsumer<T>(){
			parameters	= NULL;
			graph		= NULL;
			length		= 0;
		}

		AbstractGraph* getGraph(){
			return graph;
		}

		virtual void init(ObservationSet<T>*) = 0;
		virtual void store(ObservationSet<T>*) = 0;
		virtual void clear() = 0;

		virtual unsigned int* getCounts() = 0;
		virtual unsigned int* getCountsGPU() = 0;

		virtual unsigned int getTotalParameterCount() = 0;

		virtual void* getConfig() = 0;
		virtual void* getConfigGPU() = 0;

		virtual void synchronizeParameters() = 0;

		virtual T* getParametersGPU() = 0;

		virtual unsigned int getGPUMemUsage() = 0;

		virtual void saveToFile(std::string) = 0;
		virtual void loadFromFile(std::string) = 0;

		T* getParameters(){
			return parameters;
		};

		unsigned int getParameterCount(){
			return length;
		};

		unsigned int getDim(){
			return length;
		};

		void setInitStepsize(float s){
			init_stepsize = s;
		}

		void setInitWeight(float w){
			init_weight = w;
		}
};

# include <ext/hash_map>

namespace __gnu_cxx {
  template<>
  struct hash<ParameterDescriptor> {
    inline size_t operator()(const ParameterDescriptor &cd) const {
		unsigned long a = (((unsigned long)cd.a & (unsigned long)0x00000000000FFFFF));
		unsigned long i = (((unsigned long)cd.i & (unsigned long)0x00000000000FFFFF)<<20);
		unsigned long j = (((unsigned long)cd.j & (unsigned long)0x00000000000FFFFF)<<40);
		return (unsigned long)(a|i|j);
    };
  };
};
#endif
