// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef Optimizable_H
#define Optimizable_H

#include <malloc.h>

#include "Model.h"

enum Ability {Gradient, Hessian, Hv};

template <class T>
class Optimizable{

	protected:
		T* gradient;
		T* hessian;
		T* hv;

		T value;

	public:
		Optimizable(){
			gradient = NULL;
			hessian  = NULL;
			hv       = NULL;
		}

		virtual T compute() = 0;
		virtual bool provides(Ability) = 0;
		virtual void computeGradient() = 0;
		virtual void computeHessian() = 0;
		virtual void computeHv() = 0;

		virtual Model<T>* getModel() = 0;

		T* getGradient(){ return gradient; };
		T* getHessian(){ return hessian; };
		T* getHv(){ return hv; };
		T getValue(){ return value; };
};
#endif
