// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef Dictionary_H
#define Dictionary_H

#include <string>
#include <map>
#include <fstream>
#include <sstream>

#include "misc/strtokenizer.h"

//#include "old/smap.h"

# include <ext/hash_map>
using __gnu_cxx::hash_map;
namespace __gnu_cxx {
  template<>
  struct hash<std::string> {
    hash<char*> h;
    inline size_t operator()(const std::string &s) const { return h(s.c_str());
    };
  };
};

class Dictionary{

	private:
		__gnu_cxx::hash_map<std::string, unsigned int> tokens;
		__gnu_cxx::hash_map<unsigned int, std::string> invTokens;

		std::string* empty;

	public:

		Dictionary(){
			empty = new std::string("[NIL]");
		}

		~Dictionary(){
			delete empty;
		}

		unsigned int addToken(std::string* w){
			const unsigned int s = 1+tokens.size();
			if(!isToken(w)){
				//std::cout << "Dictionary.addToken(" << *w << ") = " << s << std::endl;
				tokens[*w] = s;
				invTokens[s] = *w;
			}
			return tokens[*w]-1;
		};

		std::string* getToken(unsigned int i){
		if(isToken(i)) return &invTokens[i+1];
			else return empty;
		}

		int getTokenId(std::string* w){
			if(isToken(w)) return (tokens[*w]-1);
			return -1;
		}

		bool isToken(std::string* w){
			return !(tokens.find(*w) == tokens.end());
		};

		bool isToken(unsigned int w){
			return !(invTokens.find(w+1) == invTokens.end());
		};

		unsigned int loadFromFile(std::string s, unsigned int peek){

			std::fstream ff(s.c_str());
			ff.seekg(peek);
			std::string line;
			std::string mark("[ENDDICT]");
			std::getline(ff, line);
			unsigned int cc = 0;

			while(line.compare(mark)!=0){
				strtokenizer* tok = new strtokenizer(&line, " \t");
//				ff << ii->first << '\t' << ii->second << std::endl;
				stringstream sstr(*(tok->token(1)));
				sstr >> cc;
				//std::cout << "TOKEN: " << *(tok->token(0)) << '\t' << cc << std::endl;
				tokens[*(tok->token(0))] = cc;
				invTokens[cc] = *(tok->token(0));
				delete tok->token(0);
				delete tok->token(1);
				delete tok;
				std::getline(ff, line);
			}
			//std::cout << "FOUND END!" << std::endl;
			unsigned int adr = ff.tellg();
			ff.close();
			return adr;
		}

		void addToFile(std::string s){
			__gnu_cxx::hash_map<std::string, unsigned int>::const_iterator ii = tokens.begin();

			std::fstream ff(s.c_str());
			ff.seekp(0, ios_base::end);
			for(; ii != tokens.end(); ii++){
				ff << ii->first << '\t' << ii->second << std::endl;
			}
			ff << "[ENDDICT]" << std::endl;
			ff.close();
		}

		unsigned int getSize(){
			return tokens.size();
		}
};
#endif
