// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef StructuredObservation_H
#define StructuredObservation_H

#include <malloc.h>
#include <string>
#include <iostream>
#include <sstream>
#include <set>
#include <map>
#include <vector>
#include <assert.h>

#include "AbstractLayer/AbstractGraph.h"
#include "AbstractLayer/AttributeConsumer.h"

#include "misc/strtokenizer.h"

#include "Dictionary.h"
#include "ParameterDescriptor.h"


template <class T>
class StructuredObservation{

	private:
		unsigned int nodeCount;
		unsigned int length;
		unsigned int nodesInFactors;
		unsigned int nils;

		Dictionary* attDict;
		Dictionary* classDict;

		unsigned int* observed;
		unsigned int* map;
		unsigned int* classes;
		T* values;

		std::map<unsigned int, std::vector<ParameterDescriptor>* >* waitingNodes;

		bool updateDicts;

		//std::string *edgeAtt;

	public:
		StructuredObservation(Dictionary* a, Dictionary* c, bool u){
			//std::cout << "StructuredObservation(" << a << ", " << c << ", " << u << ");" << std::endl;
			attDict     = a;
			classDict   = c;
			updateDicts = u;

			nodeCount = 0;
			length    = 0;
			nodesInFactors = 0;
			nils = 0;

			observed = NULL;
			values   = NULL;
			map      = NULL;
			classes  = NULL;

			//edgeAtt = new std::string("EDGE");
			waitingNodes = new std::map<unsigned int, std::vector<ParameterDescriptor>* >();
		};

		~StructuredObservation(){
			if(observed)	free(observed);
			if(values)		free(values);
			if(map)			free(map);
			if(classes)		free(classes);

			typename std::map<unsigned int, std::vector<ParameterDescriptor>* >::const_iterator ii = waitingNodes->begin();
			for(; ii != waitingNodes->end(); ii++){
				delete ii->second;
			}
			delete waitingNodes;
			//delete edgeAtt;
		};


		unsigned int getNodeCount(){
			return nodeCount;
		}

		unsigned int getNilCount(){
			return nils;
		}

		unsigned int getNodesInFactors(){ return nodesInFactors; }

		std::string* toString(){
			std::string* s = new string();
			unsigned int* att;

			for(unsigned int i=0; i<nodeCount; i++){
				s->append(*classDict->getToken(classes[i]));

				s->append("\t");
				att = getNodeObservations(i);
				for(unsigned int j=0; j < getNodeObservationCount(i); j++){
					s->append(*attDict->getToken(att[j]));
					s->append("\t");
				}

				s->append("\n");
			}
			return s;
		}

		unsigned int* getNodeObservations(unsigned int node){
			return &observed[map[node]];
		}

		unsigned int* getObservations(){
			return observed;
		}

		T* getValues(){
			return values;
		}

		unsigned int* getLabels(){
			return classes;
		}

		unsigned int getNodeLabel(unsigned int node){
			return classes[node];
		}

		unsigned int getNodeIndex(unsigned int node){
			return map[node];
		}

		unsigned int* getNodeMap(){
			return map;
		}

		unsigned int getNodeObservationCount(unsigned int node){
			return map[node+1]-map[node];
		}

		unsigned int getSize(){
			return length;
		}

		bool addNode(std::string* s, unsigned int& maxAtt, AttributeConsumer<T>* aC, unsigned int& totalNodes, unsigned int desiredId){

			assert(nodeCount < AbstractGraph::pbase);
			{

			T value = (T)1.0;
			unsigned int id = 0, npos;
			bool delete_w = false;
			bool nil_label = false;

			strtokenizer* tok = new strtokenizer(s, " \t");

			std::string *w = tok->token(0), *v, *t;

			if(w->size() == 3){
				if((*w)[0] == 'N' && (*w)[1] == 'I' && (*w)[2] == 'L'){
					std::cout << "NIL-Label: " << *s << std::endl;
					nil_label = true;
					delete w;
				}
			}

			if(!nil_label){

			if(!classDict->isToken(w) && updateDicts){
				//std::cout << "added Label: " << *w << std::endl;
				classDict->addToken(w);
			}
			else delete_w = true;

			if(!classDict->isToken(w)){
				if(w) delete w;
				delete tok;
				return false;
			}

			if(!classes) classes = (unsigned int*) malloc(sizeof(unsigned int));
			else classes = (unsigned int*) realloc(classes, (nodeCount+1)*sizeof(unsigned int));
			classes[nodeCount] = classDict->getTokenId(w);

			//std::cout << classes[nodeCount] << std::endl;
			} //!nil_label

			if(!map) map = (unsigned int*) malloc(2*sizeof(unsigned int));
			else map = (unsigned int*) realloc(map, (nodeCount+nils+2)*sizeof(unsigned int));

			map[nodeCount+nils] = length;

			if(!nil_label){
				nodeCount++;
			}else{
				nils++;
			}

			if(delete_w){
				delete w;
				delete_w = false;
			}

			for(unsigned int i=1; i<tok->count_tokens(); i++){

				//if(i==0) w = new std::string("EDGE");
				//else w = tok->token(i);

				w = tok->token(i);

				value = 1.0;
/*
				if(w->at(0) == 'N'){

					npos = w->find_first_of('=');
					
					t = w;
					w = new std::string( t->substr(0, npos).c_str() );
					v = new std::string( t->substr(npos+1, t->size()-npos).c_str() );
					delete t;
					std::istringstream ss(v->c_str());
					ss >> value;
					delete v;
				}
*/				

				if(!attDict->isToken(w) && updateDicts){
					attDict->addToken(w);
				}
				else delete_w = true;

				if(attDict->isToken(w)){
					if(!observed) observed = (unsigned int*) malloc(sizeof(unsigned int));
					else observed = (unsigned int*) realloc(observed, (length+1)*sizeof(unsigned int));

					id = attDict->getTokenId(w);
					//std::cout << "id = " << id << "\tw = " << *w << std::endl;
					assert(id>=0);
					//if(id == 105) std::cout << "Observed \"" << *(attDict->getToken(id)) << "\", id = " << id << std::endl;
					observed[length] = id;
					if(!nil_label) nodesInFactors = aC->addAttribute(id, totalNodes, nils+nodeCount-1, nils+nodeCount, desiredId, classes, waitingNodes, updateDicts);
					//seen->insert(id);
					if(id>maxAtt) maxAtt = id;
					//std::cout << "maxAtt = " << maxAtt << std::endl;

					if(!values) values = (T*) malloc(sizeof(T));
					else values = (T*) realloc(values, (length+1)*sizeof(T));
					values[length++] = value;
				}

				if(delete_w){
					delete w;
					delete_w = false;
				}
			}

			map[nodeCount] = length;

			delete tok;
			//delete edgeAtt;

			if(!nil_label) return true;
			else return false;

			}//else{
			//	return false;
			//}
		}
};

#endif
