// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef Graph_H
#define Graph_H

#include "AbstractLayer/AbstractGraph.h"

class Graph : public AbstractGraph{
	protected:

		using AbstractGraph::adjacency;
		using AbstractGraph::sources;
		using AbstractGraph::sinks;
		using AbstractGraph::edgeCount;
		using AbstractGraph::edges;
		using AbstractGraph::edgeMap;
		using AbstractGraph::updateEdges;
		using AbstractGraph::updateEdgeMap;

	public:

		Graph() : AbstractGraph() {};
		~Graph(){
			for(std::vector<std::set<unsigned int>* >::const_iterator ii = adjacency.begin(); ii != adjacency.end(); ii++)
				delete *ii;
		};

		bool isDynamic(){ return false; };

		// setNodeCount() wird nur für dynamische Graphen benötigt.
		void setNodeCount(unsigned int){};

		void addNode(){
			adjacency.push_back( new std::set<unsigned int>() );
			updateEdges = true;
			updateEdgeMap = true;
		};

		unsigned int getNodeCount(){
			return adjacency.size();
		}

		bool isNode(unsigned int v){
			return (v < getNodeCount());
		}

		void addEdge(unsigned int u, unsigned int v, bool undirected = true){
			if((u < adjacency.size()) && (v < adjacency.size())){
				adjacency.at(u)->insert(v);
				edgeCount++;
				if(undirected){
					adjacency.at(v)->insert(u);
					edgeCount++;
				}
				updateEdges = true;
				updateEdgeMap = true;
			}
		}

		bool isEdge(unsigned int u, unsigned int v){
			if((u < adjacency.size()) && (v < adjacency.size()))
				return (adjacency.at(u)->find(v) != adjacency.at(u)->end());
			else
				return false;
		}

		unsigned int* getEdges(){
			if(updateEdges){
				unsigned int* E = (unsigned int*) malloc(edgeCount*sizeof(unsigned int));

				unsigned int i = 0;

				std::vector<std::set<unsigned int>* >::iterator ii = adjacency.begin();
				std::set<unsigned int>::iterator jj;

				for(; ii != adjacency.end(); ii++){
					jj = (*ii)->begin();
					for(; jj != (*ii)->end(); jj++){
						E[i++] = (*jj);
					}
				}

				if(edges) free(edges);
				edges = E;
				updateEdges = false;
				return E;
			}else{
				return edges;
			}
		}

		unsigned int* getEdgeMap(){
			if(updateEdgeMap){
				unsigned int* M = (unsigned int*) malloc(adjacency.size()*sizeof(unsigned int));

				std::vector<std::set<unsigned int>* >::iterator ii = adjacency.begin();

				unsigned int i = 0, j = 0;

				for(; ii != adjacency.end(); ii++){
					M[i] = j;
					j = (*ii)->size();
					i++;
				}
				if(edgeMap) free(edgeMap);
				edgeMap = M;
				updateEdgeMap = false;
				return M;
			}else{
				return edgeMap;
			}
		}
};
#endif
