// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef LinearChain_H
#define LinearChain_H

#include <algorithm>

#include "AbstractLayer/AbstractGraph.h"

class LinearChain : public AbstractGraph{

	private:

		using AbstractGraph::allocated;
		using AbstractGraph::maxNodes;
		using AbstractGraph::maxEdges;
		using AbstractGraph::edges;
		using AbstractGraph::edgeMap;
		using AbstractGraph::factors;

		using AbstractGraph::edgeCount;
		using AbstractGraph::nodeCount;
		using AbstractGraph::Fcount;
		using AbstractGraph::delta_max;
		using AbstractGraph::nodesInFactors;
		using AbstractGraph::nodeFactors;
		using AbstractGraph::nodeFactorsIdx;
		using AbstractGraph::nodeFactorsMap;

		bool needsRebuild;

	public:

		LinearChain(unsigned int n = 10) : AbstractGraph(){

			delta_max  = 2;
			maxNeighbours = 3;

			needsRebuild = true;

			build(n);
		};

		~LinearChain(){
			free(edges);
			free(edgeMap);
			free(factors);
			free(nodeFactors);
			free(nodeFactorsIdx);
			free(nodeFactorsMap);
		};

		// Der Linear Chain Graph ist dynamisch in der Anzahl der Knoten.
		bool isDynamic(){ return true; };
		void addNode(){};
		void addEdge(unsigned int u,unsigned int v, bool b){};

		void setNodeCount(unsigned int n){};

		void build(unsigned int n){
			nodeCount = n;
			edgeCount = n-1;

			if(nodeCount > allocated){

				needsRebuild = true;

				if(nodeCount > maxNodes) maxNodes = nodeCount;
				if(edgeCount > maxEdges) maxEdges = edgeCount;

				Fcount    = maxNodes + maxEdges;
				nodesInFactors = (3*maxNodes)-1;

				if(Fcount > Fmax) Fmax = Fcount+1;

				if(!allocated){
					edges   	= (unsigned int*) malloc(edgeCount		* sizeof(unsigned int));
					edgeMap 	= (unsigned int*) malloc((nodeCount+1)	* sizeof(unsigned int));
					factors 	= (FactorNode*)malloc(Fcount	* sizeof(FactorNode));

					nodeFactors = (unsigned long*)malloc(nodesInFactors	* sizeof(unsigned long));
					nodeFactorsIdx = (unsigned long*)malloc(nodesInFactors	* sizeof(unsigned long));
					nodeFactorsMap = (unsigned int*)malloc((nodeCount+1) * sizeof(unsigned int));

					edges[0]   = 0;
					edgeMap[0] = 0;
					edgeMap[1] = 0;

					allocated  = 1;
				}else{
					edges   = (unsigned int*)realloc(edges, edgeCount * sizeof(unsigned int));
					edgeMap = (unsigned int*)realloc(edgeMap, (nodeCount+1) * sizeof(unsigned int));
					factors = (FactorNode*)realloc(factors, Fcount * sizeof(FactorNode));

					nodeFactors = (unsigned long*)realloc(nodeFactors, nodesInFactors	* sizeof(unsigned long));
					nodeFactorsIdx = (unsigned long*)realloc(nodeFactorsIdx, nodesInFactors	* sizeof(unsigned long));
					nodeFactorsMap = (unsigned int*)realloc(nodeFactorsMap, (nodeCount+1) * sizeof(unsigned int));

					allocated = nodeCount;
				}


				unsigned int ccount = 0;
				unsigned int inFactors = 0;

				for(int i=0; i<nodeCount; i++){
						if(i){
							edges[i-1] = i-1;
							edgeMap[i] = i-1;
						}

						nodeFactorsMap[i] = inFactors;

						unsigned int myFactors = 1;

						// Node-Factor existiert immer
						// hier einfügen..

						factors[ccount].c = AbstractGraph::baseEncode(i+1, 0, AbstractGraph::pbase);
						factors[ccount].o = AbstractGraph::baseEncode(i+1, 0, AbstractGraph::pbase);
						factors[ccount].m = AbstractGraph::baseEncode(myFactors++, 0, AbstractGraph::pbase);
						factors[ccount].t = 0;

						nodeFactors[inFactors++] = factors[ccount++].c;

						// Linker Nachbar existiert
						if(i>0){

							factors[ccount].c = AbstractGraph::baseEncode(i+1, i, AbstractGraph::pbase);
							factors[ccount].o = AbstractGraph::baseEncode(i+1, 0, AbstractGraph::pbase);
							factors[ccount].t = 1;

							if(i==1) factors[ccount].m = AbstractGraph::baseEncode(myFactors++, 2, AbstractGraph::pbase);
							else factors[ccount].m = AbstractGraph::baseEncode(myFactors++, 3, AbstractGraph::pbase);

							nodeFactors[inFactors++] = factors[ccount++].c;
						}

						// Rechter Nachbar existiert
						if(i<nodeCount-1){
							// Faktor wird von rechtem Nachbarn in factors[] eingefügt

							nodeFactors[inFactors++] = AbstractGraph::baseEncode(i+2, i+1, AbstractGraph::pbase);
							myFactors++;
						}

				}

				nodeFactorsMap[nodeCount] = inFactors;
				edgeMap[nodeCount] = edgeCount;
				std::sort(factors, factors+Fcount);

				rebuildFactors();
			}
		};

		unsigned int getTemplateCount(){
			return 2;
		}

		void rebuildFactors(){

			if(needsRebuild){
				//std::cout << "*** REBUILD ***" << std::endl;
				std::copy(nodeFactors, nodeFactors+nodesInFactors, nodeFactorsIdx);

				for(unsigned long i=0; i<Fcount; i++){
					//std::cout << std::endl << factors[i].c << '\t' << factors[i].o << '\t' << factors[i].m << '\t' << std::endl;
					//std::cout << "Replacing " << factors[i].c << " with " << i << std::endl;
					std::replace(nodeFactorsIdx, nodeFactorsIdx+nodesInFactors, factors[i].c, i);
				}
				//for(unsigned long i=0; i<nodesInFactors; i++){
				//	std::cout << "Faktor " << i << '\t' << nodeFactorsIdx[i] << std::endl;
				//}
				needsRebuild = false;
			}

		};

		bool isEdge(unsigned int u, unsigned int v){
			return (v!=0) && (v<nodeCount) && (u<nodeCount) && (u == (v-1));
		}
};
#endif
