// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef LogLikelihood_CUH
#define LogLikelihood_CUH

template <class T>
__global__ void gradientLLKernel(unsigned int dim, T* parameter, T* grad, T* expect, unsigned int* empExpect, void* conf){

	unsigned int threadID = blockIdx.x*blockDim.x+threadIdx.x;
	T eE, mE, t;

	while(threadID < dim){

		eE = (T)empExpect[threadID];
		mE = (T)expect[threadID];
		t = eE - mE;
		grad[threadID] = -t;

		threadID += blockDim.x*gridDim.x;
	}
}

template <class T>
void gradientLLGPU(unsigned int dim, T* parameter, T* grad, T* expect, unsigned int* counts, void* conf, unsigned int bb, unsigned int tb){

	dim3 grid(bb, 1, 1);
	dim3 block(tb, 1, 1);

	gradientLLKernel<T><<<grid, block>>>(dim, parameter, grad, expect, counts, conf);

	cudaFree(conf);
}

template void gradientLLGPU<float>(unsigned int,float*,float*,float*,unsigned int*,void*, unsigned int bb, unsigned int tb);
template void gradientLLGPU<double>(unsigned int,double*,double*,double*,unsigned int*,void*, unsigned int bb, unsigned int tb);

template <class T>
T valueLLGPU(unsigned int dim, T* parameter, unsigned int N, T* logProps){
	dim3 dimGrid(128, 1, 1);
	dim3 dimBlock(128, 1, 1);
	dim3 dimOne(1, 1, 1);
	dim3 dimOneBlock(64, 1, 1);

	const unsigned int smem = 128 * sizeof(T);
	T *buffer;
	cudaMalloc((void**)&buffer, smem);

	reduceSumKernel<T><<<dimGrid, dimBlock, smem>>>(buffer, logProps, N);
	reduceOneBlock<T><<<dimOne, dimOneBlock, smem>>>(buffer);

	T f;
	cudaMemcpy(&f, buffer, sizeof(T), cudaMemcpyDeviceToHost);

	cudaFree(buffer);

	return -f;
}

template float valueLLGPU<float>(unsigned int dim, float* parameter, unsigned int N, float* logProps);
template double valueLLGPU<double>(unsigned int dim, double* parameter, unsigned int N, double* logProps);
#endif
