// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef LogLikelihood_H
#define LogLikelihood_H

#include "AbstractLayer/Optimizable.h"
#include "AbstractLayer/InferenceEngine.h"

#include "DataLayer/ObservationSet.h"

template <class T> void gradientLLGPU(unsigned int,T*,T*,T*,unsigned int*,void*, unsigned int bb, unsigned int tb);
template <class T> T valueLLGPU(unsigned int, T*, unsigned int, T*);

template <class T>
class LogLikelihood : public Optimizable<T>{

	private:
		ObservationSet<T>* observations;
		InferenceEngine<T>* density;

		unsigned int gradAlloc;
		unsigned int blockBase;
		unsigned int threadBase;

		using Optimizable<T>::gradient;
		using Optimizable<T>::hessian;
		using Optimizable<T>::hv;
		using Optimizable<T>::value;

	public:
		LogLikelihood(InferenceEngine<T>* d, ObservationSet<T>* s, unsigned int bb = 240, unsigned int tb = 128) : Optimizable<T>(){
			density = d;
			observations = s;
			gradAlloc = 0;
			blockBase = bb;
			threadBase = tb;
			value = (T)0.0;
			gradient = 0;
		}

		~LogLikelihood(){
			cudaFree(gradient);

		}

		void setObservations(ObservationSet<T>* s){ observations = s; };
		void setDensity(InferenceEngine<T>* d){ density = d; };

		T compute(){

			density->computeMarginals(observations);
			computeGradient();

			//value = valueLLGPU(density->getModel()->getDim(), density->getModel()->getParametersGPU(), observations->getSize(), density->getLogPropsGPU());
			//std::cout << "-log(L) = " << setprecision(8) << value << std::endl;

			return value;
		}

		void computeGradient(){
			cudaRealloc(&gradient, gradAlloc, density->getModel()->getDim());
		

			//std::cout << "call gradientLLGPU " << density->getModel()->getDim() << std::endl;
			gradientLLGPU<T>(density->getModel()->getDim(), density->getModel()->getParametersGPU(), gradient, density->getExpectations(), density->getModel()->getCountsGPU(), density->getModel()->getConfigGPU(), blockBase, threadBase);
		}

		Model<T>* getModel(){ return density->getModel(); };

		void computeHessian(){};
		void computeHv(){};

		bool provides(Ability a){
			switch(a){
				case Gradient: return true;
				case Hessian: return false;
				case Hv: return false;
				default: return false;
			}
		}
};
#endif
