// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef MEMUTIL_CUH
#define MEMUTIL_CUH

/*********************************************************************/

template<class T>
struct SharedMemory
{
    __device__ inline operator       T*()
    {
        extern __shared__ int __smem[];
        return (T*)__smem;
    }

    __device__ inline operator const T*() const
    {
        extern __shared__ int __smem[];
        return (T*)__smem;
    }
};


// specialize to avoid unaligned memory access compile errors
template<>
struct SharedMemory<double>
{
    __device__ inline operator double*()
    {
        extern __shared__ double __smem_d[];
        return (double*)__smem_d;
    }

    __device__ inline operator const double*() const
    {
        extern __shared__ double __smem_d[];
        return (double*)__smem_d;
    }
};
/*********************************************************************/

template <class T>
__global__ void cudaMemsetKernel(T* array, T value, unsigned int n){

	unsigned int threadID = blockIdx.x*blockDim.x+threadIdx.x;

	//cuPrintfRestrict(0, 0);
	//cuPrintf("array[%d] = %f\n", threadID, value);

	while(threadID < n){
		array[threadID] = value;
		threadID += blockDim.x*gridDim.x;
	}
}
/*********************************************************************/

template <class T>
__global__ void clearGPUKernel(T* array, unsigned int n){

	unsigned int threadID = blockIdx.x*blockDim.x+threadIdx.x;

	while(threadID < n){
		array[threadID] = (T)0;
		threadID += gridDim.x*blockDim.x;
	}
}

template <class T>
void clearGPU(T* array, unsigned int n){
	dim3 clearGrid(240, 1, 1);
	dim3 clearBlock(128, 1, 1);

	clearGPUKernel<T><<<clearGrid, clearBlock>>>(array, n);
}

template void clearGPU<float>(float* array, unsigned int n);
template void clearGPU<double>(double* array, unsigned int n);
#endif

