// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef Reduction_H
#define Reduction_H

///////////////////////////////////////////////////////////////////////////////
// Reduction-Kernels
///////////////////////////////////////////////////////////////////////////////

template <class T>
__device__ void smemReduce(volatile T* sdata, unsigned int tid){

	if (tid < 32){
		sdata[tid] += sdata[tid + 32];
		sdata[tid] += sdata[tid + 16];
		sdata[tid] += sdata[tid +  8];
		sdata[tid] += sdata[tid +  4];
		sdata[tid] += sdata[tid +  2];
		sdata[tid] += sdata[tid +  1];
	}
}

template <class T>
__global__ void reduceSumKernel(T* g_odata, T* g_idata, unsigned int n){

	volatile T *sdata = SharedMemory<T>();

	unsigned int tid = threadIdx.x;
	unsigned int i = blockIdx.x*(2*128) + threadIdx.x;
	unsigned int gridSize = 2*128*gridDim.x;
	sdata[tid] = (T)0.0;

	unsigned int idx = 0;

	while (i < n){
		idx = i;
        sdata[tid] += g_idata[idx];

 		if (i + 128 < n){
			idx = i + 128;
			sdata[tid] += g_idata[idx];
		}
		i += gridSize;
	}
	__syncthreads();

	if (tid <  64) sdata[tid] += sdata[tid +  64];
	__syncthreads();

	smemReduce(sdata, tid);
	if (tid == 0) g_odata[blockIdx.x] = sdata[0];
}

template <class T>
__global__ void reduceSumOfQuadsKernel(T* g_odata, T* g_idata, unsigned int n){

	volatile T *sdata = SharedMemory<T>();

	unsigned int tid = threadIdx.x;
	unsigned int i = blockIdx.x*(2*128) + threadIdx.x;
	unsigned int gridSize = 2*128*gridDim.x;
	sdata[tid] = (T)0.0;
	T temp = (T)0.0;

	while (i < n){
		temp = g_idata[i];
		sdata[tid] += (temp*temp);
		if (i + 128 < n){
			temp = g_idata[i+128];
			sdata[tid] += (temp*temp);
		}
		i += gridSize;
	}
	__syncthreads();

	if (tid <  64) sdata[tid] += sdata[tid +  64];
	__syncthreads();

	smemReduce(sdata, tid);
	if (tid == 0) g_odata[blockIdx.x] = sdata[0];
}

template <class T>
__global__ void reduceOneBlock(T *data){

	volatile T *sdata = SharedMemory<T>();

	const unsigned int tid = threadIdx.x;
	const unsigned int i = threadIdx.x;
	sdata[tid] = data[i] + data[i+64];
	__syncthreads();
	smemReduce(sdata, tid);

	if (tid == 0) data[tid] = sdata[tid];
}
#endif
