// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef SGD_CUH
#define SGD_CUH

template <class T>
__global__ void parameterUpdateSGDKernel(T* parameter, T* gradient, T eta, unsigned int n){

	unsigned int threadID = blockIdx.x*blockDim.x+threadIdx.x;

	while(threadID < n){

		parameter[threadID] = parameter[threadID] - ( eta * gradient[threadID] );

		threadID += blockDim.x*gridDim.x;
	}
}

template <class T>
void parameterUpdateSGDGPU(T* parameter, T* gradient, T eta, unsigned int n, unsigned int bb, unsigned int tb){

	dim3 dimGridB(bb, 1, 1);
	dim3 dimBlockB(tb, 1, 1);

	parameterUpdateSGDKernel<T><<<dimGridB, dimBlockB>>>(parameter, gradient, eta, n);
}

template void parameterUpdateSGDGPU<float>(float* parameter, float* gradient, float eta, unsigned int n, unsigned int bb, unsigned int tb);
template void parameterUpdateSGDGPU<double>(double* parameter, double* gradient, double eta, unsigned int n, unsigned int bb, unsigned int tb);
#endif
