// Copyright (C) 2011 Nico Piatkowski <nico.piatkowski@tu-dortmund.de>, All rights reserved 
#ifndef SGD_H
#define SGD_H

#include <cublas.h>

#include "AbstractLayer/Optimizer.h"

template <class T> void parameterUpdateSGDGPU(T* parameter, T* gradient, T eta, unsigned int n, unsigned int bb, unsigned int tb);

template <class T>
class SGD : public Optimizer<T>{

	private:

		T eta;
		unsigned int blockBase;
		unsigned int threadBase;

		using Optimizer<T>::objective;

		void iterateSingle(){
			objective->compute();
			T e = eta;// / cublasDnrm2(objective->getModel()->getDim(), objective->getGradient(), 1);

			parameterUpdateSGDGPU(objective->getModel()->getParametersGPU(), objective->getGradient(), e, objective->getModel()->getDim(), blockBase, threadBase);
			objective->getModel()->synchronizeParameters();
		};

	public:

		SGD(T e, Optimizable<T>* o, unsigned int bb = 240, unsigned int tb = 128) : Optimizer<T>(o){
			eta = e;
			blockBase = bb;
			threadBase = tb;
		};

		~SGD(){

		};

		void setStepsize(T e){
			std::cout << "Setting eta to " << e << std::endl;
			eta = e;
		}

		void iterate(int n = 1){
			for(unsigned int i=0; i<n; i++){
				iterateSingle();
			}
		};

};
#endif
